package ca.aura.algorithms;

import java.util.Iterator;

import ca.aabeka.math.NormalDistribution;
import ca.aabeka.util.Graph;
import ca.aabeka.util.Hashtable;
import ca.aabeka.util.Vector;

/**
 * 
 * @author aforward
 *
 */
public class AuthoritativeSources implements AuraAlgorithm  {

  private int DEFAULT_ITERATIONS = 20;

  private Hashtable auth_;
  private Hashtable hub_;
  int k_;

  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default is given a whole bunch of artefacts and the entire depot
   * 
   */

  public AuthoritativeSources(Vector artefacts, Graph associations) {
    init(artefacts,associations, DEFAULT_ITERATIONS);
  }

  /**
   * 
   * also specify how often to do it
   * 
   */
  public AuthoritativeSources(Vector artefacts, Graph associations, int k) {
    init(artefacts,associations, k);
  }  
  

  private void init(Vector rootSet, Graph graph, int k) {
    k_ = k;
    
    Graph base = buildBase(rootSet, graph);
    
    auth_ = buildInitialHash(base);
    hub_ = buildInitialHash(base);
    
    for (int i=0; i<k_; i++) {
    
      Iterator points;
      points = base.points();
      while (points.hasNext()) {
        updateAuthority(points.next(), base, auth_, hub_);
      }

      points = base.points();
      while (points.hasNext()) {
        updateHub(points.next(), base, auth_, hub_);
      }

      normalize(auth_);
      normalize(hub_);        
    }
    

  }


  //`````````````````````
  // inteface
  //`````````````````````

  /**
   * 
   * @see ca.aura.algorithms.RateAlgorithm#getResults()
   * 
   */
  
  public Hashtable getResults() {
    return auth_;  
  }

  /**
   * 
   * build the base vector
   * 
   * The entire graph may have
   * a -> x,y
   * b -> x
   * z -> b
   * y -> z
   * y -> w
   * w -> x
   * 
   * root = [ a, b ]
   * base = [ a, b, y, z ]
   * 
   * We are ignoring becase they are outside of our base
   * y -> w
   * w -> x
   * 
   */
  
  
  public Graph buildBase(Vector rootSet, Graph graph) {

    Vector base = new Vector(false);
    base.addAll(rootSet);
    
    Iterator iter;
    
    iter = rootSet.iterator();
    while (iter.hasNext()) {
      Object next = iter.next();
      base.addAll(graph.outReferencesFrom(next));
      base.addAll(graph.inReferenceesTo(next));      
    }
    
    return graph.getSubGraph(base);
  }
  
  /**
   * 
   * get the authoritative value of point p
   * 
   * 
   * get all q that points to p
   * 
   * q1 -> p
   * q2 -> p
   * q3 -> p
   * 
   * then sum all the hub values of those q's
   * 
   */

  public void updateAuthority(Object p, Graph g, Hashtable auth, Hashtable hub) {
    
    double d = 0.0;
    Iterator qs = g.inReferenceesTo(p);
    while (qs.hasNext()) {
      d += ((Double)hub.get(qs.next())).doubleValue();
    }

    auth.put(p, new Double(d));
  }


  /**
   * 
   * get the hubness value of point q
   * 
   * 
   * get all p's that q points to
   * 
   * q -> p1
   * q -> p2
   * q -> p3
   * 
   * then sum all the auth values of those p's
   * 
   */

  public void updateHub(Object q, Graph g, Hashtable auth, Hashtable hub) {
    
    double d = 0.0;
    
    Iterator ps = g.outReferencesFrom(q);
    while (ps.hasNext()) {
      d += ((Double)auth.get(ps.next())).doubleValue();
    }
    
    hub.put(q, new Double(d));
  }

  /**
   * 
   * build the initial hashtables for the hub and authority values
   * 
   */
  public Hashtable buildInitialHash(Graph g) {
    
    Hashtable hash = new Hashtable();
    
    Iterator points = g.points();
    while (points.hasNext()) {
      hash.put(points.next(), new Double(1.0));    
    }  
    return hash;
  }

  /**
   * 
   * normalize a hashtable of values :-)
   * 
   */
  
  public void normalize(Hashtable hash) {
    
    Vector values = new Vector();
    values.addAll(hash.getValues());
    NormalDistribution ndist = new NormalDistribution(values);
    
    Iterator iter = hash.getKeys();
    while (iter.hasNext()) {
      Object key = iter.next();
      double x = ((Double)hash.get(key)).doubleValue();
      hash.put(key, new Double(ndist.normalize(x)));  
    }
  }
  
}
