package ca.aura.algorithms;

import java.util.Date;
import java.util.Iterator;

import ca.aabeka.util.DateAndTime;
import ca.aabeka.util.Debug;
import ca.aabeka.util.Hashtable;
import ca.aura.algorithms.lrfu.F;
import ca.aura.algorithms.lrfu.SimpleF;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.reports.Report;
import ca.aura.artefacts.usage.Usage;

/**
 * 
 * @author aforward
 *
 */
public class LRFU implements AuraAlgorithm  {

  Hashtable CRF_;
  Hashtable lastAccessed_;
  F f_;

  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */
  
  public LRFU(Report report) {
    init(report, new SimpleF());  
  }

  /**
   * 
   * default
   * 
   */
  
  public LRFU(Report report, F f) {
    init(report, f);  
  }

  private void init(Report report, F f) {
    
    f_ = f;
    
    initialCRF(report);
    Iterator iter = report.getArtefacts();
    while (iter.hasNext()) {
      Artefact artefact = (Artefact)iter.next();
      
      Iterator usageIter = report.getUsage(artefact);
      while (usageIter.hasNext()) {
        Usage usage = (Usage)usageIter.next();
        updateCRF(artefact,usage);
      }
    }
    finalizeCRF(report);
  }

  //`````````````````````
  // interface
  //`````````````````````

  /**
   * 
   * @see ca.aura.algorithms.RateAlgorithm#getResults()
   * 
   */
  public Hashtable getResults() {
    return CRF_;  
  }

  /**
   * 
   * get vector of sorted results
   * 
   */
  
  public Iterator sortedResults() {
    return new ResultIterator(this);
  }



  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * init the CRF numbers
   * 
   */

  protected void initialCRF(Report report) {
    lastAccessed_ = new Hashtable();
    CRF_ = new Hashtable();
    Iterator iter = report.getArtefacts();
    while (iter.hasNext()) {
      CRF_.put(iter.next(), new Double(0));   
    }
  }

  /**
   * 
   * update the crf number
   * 
   */
  
  protected void updateCRF(Artefact artefact, Usage usage) {
    
    Date newDate = usage.getDate();
    
    if (!lastAccessed_.containsKey(artefact)) {
      setCRF(artefact,F(0));
      lastAccessed_.put(artefact,newDate);
      return;
    }    
    
    Date oldDate = (Date)lastAccessed_.get(artefact);
    lastAccessed_.put(artefact,newDate);   

    int dt = getDeltaT(oldDate, newDate);
    double newCRF = F(0) + F(dt)*getCRF(artefact);
    setCRF(artefact,newCRF);
    
  }

  /**
   * 
   * finalize the CRF numbers
   * 
   */

  protected void finalizeCRF(Report report) {
    Iterator iter = report.getArtefacts();
    while (iter.hasNext()) {
      Artefact artefact = (Artefact)iter.next();
      
      int dT = getDeltaT((Date)lastAccessed_.get(artefact),report.getDate());
      double oldCRF = getCRF(artefact);
      double newCRF = F(dT)*oldCRF;
      setCRF(artefact,newCRF);
    }
  }

  /**
   * 
   * get the age of artefact on a certain day
   * 
   */

  protected int getDeltaT(Date start, Date end) {
    int dt = DateAndTime.getAgeInDays(start,end); 
    return dt; 
  }

  /**
   * 
   * get the last CRF value
   * 
   */

  public double getCRF(Artefact artefact) {
    return ((Double)CRF_.get(artefact)).doubleValue();  
  }

  /**
   * 
   * set the CRF value
   * 
   */

  public void  setCRF(Artefact artefact, double crf) {
    CRF_.put(artefact, new Double(crf));
  }

  /**
   * 
   * get the F value
   * 
   */
  public double F(int dt) {
    return f_.calculate(dt);  
  }


}
