package ca.aura.xml;
import java.util.Iterator;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefacts.AbstractArtefact;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.Aura;
import ca.aura.artefacts.Description;
import ca.aura.artefacts.Doc;
import ca.aura.artefacts.JavaDoc;
import ca.aura.artefacts.JunitReport;
import ca.aura.artefacts.Location;
import ca.aura.artefacts.SourceCode;
import ca.aura.artefacts.Webpage;

/**
 * 
 * @author aforward
 *
 */
public class ArtefactXmlConverter implements ObjectXmlConverter {


  private static final String DOC_TYPE = "doc";
  private static final String SOURCECODE_TYPE = "sourcecode";
  private static final String WEBPAGE_TYPE = "webpage";
  private static final String JUNIT_REPORT_TYPE = "junitreport";
  private static final String JAVADOC_TYPE = "javadoc";  

  private static final String DEFAULT_TYPE = DOC_TYPE;



  private static final String DEFAULT_ID = "?!?";

  private static final String ELEMENT_ARTEFACT = "artefact";
  private static final String ELEMENT_CONTENT = "content"; 
  private static final String ELEMENT_DESCRIPTION = "description"; 
  private static final String ELEMENT_LOCATION = "location"; 

  private static final String ATTRIBUTE_TYPE = "type";
  private static final String ATTRIBUTE_ID = "id";
  private static final String ATTRIBUTE_NAME = "name";


  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_ARTEFACT;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    
    if (ele == null) {
      return false;
    }

    return getElementName().equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof Artefact);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {

    if (!canLoad(ele)) {
      return new AbstractArtefact(DEFAULT_ID);
    }

    AbstractArtefact artefact = buildArtefact(ele);
  

    LocationXmlConverter locationXml = new LocationXmlConverter();
    Element child = ele.getChild(locationXml.getElementName());
    if (locationXml.canLoad(child)) {
      artefact.setLocation((Location)locationXml.load(child));
    }

    Element contentElement = ele.getChild(ELEMENT_CONTENT);
    if (!canLoadContent(contentElement)) {
      return artefact;
    }

    AuraXmlConverter auraXml = new AuraXmlConverter();
    Iterator auraIter = contentElement.getChildren(auraXml.getElementName()).iterator();
    while (auraIter.hasNext()) {
      Element auraElement = (Element)auraIter.next();
      if (auraXml.canLoad(auraElement)) {
        artefact.addAura((Aura)auraXml.load(auraElement));  
      }
    }
    
    return artefact;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {
    
    Element ele = new Element(getElementName());
    
    if (!canBuild(obj)) {
      return ele;  
    }

    Artefact artefact = (Artefact)obj;

    ele.setAttribute(ATTRIBUTE_ID, artefact.getId());
    ele.setAttribute(ATTRIBUTE_NAME, artefact.getName());
    
    Element location = new Element(ELEMENT_LOCATION);
    location.setText(artefact.getFile().getAbsolutePath());
    ele.addContent(location);

    DescriptionXmlConverter descrXml = new DescriptionXmlConverter();
    if (descrXml.canBuild(artefact.getDescription())) {
      ele.addContent(descrXml.build(artefact.getDescription()));  
    }

    if (artefact.numberOfAura() == 1) {
      return ele;  
    }    

    AuraXmlConverter auraXml = new AuraXmlConverter();
    Iterator auraIter = artefact.getAura();
    auraIter.next();    
    while (auraIter.hasNext()) {
      Object aura = auraIter.next();
      if (auraXml.canBuild(aura)) {
        ele.addContent(auraXml.build(aura));  
      }
    }
    return ele;
  }

  //`````````````````````
  // protected
  //`````````````````````


  /**
   * 
   * build the artefact
   * 
   */
  
  protected AbstractArtefact buildArtefact(Element ele) {
   
    Attribute attrType = ele.getAttribute(ATTRIBUTE_TYPE);
    String type = attrType != null ? attrType.getValue() : DEFAULT_TYPE;

    Attribute attrId = ele.getAttribute(ATTRIBUTE_ID);
    String id = attrId != null ? attrId.getValue() : DEFAULT_ID;

    Attribute attrName = ele.getAttribute(ATTRIBUTE_NAME);
    String name = attrName != null ? attrName.getValue() : id;
    
    AbstractArtefact artefact;    
   
    if (DOC_TYPE.equals(type)) {
      artefact =  new Doc(id); 
    } else if (SOURCECODE_TYPE.equals(type)) {
      artefact =  new SourceCode(id); 
    } else if (WEBPAGE_TYPE.equals(type)) {
      artefact =  new Webpage(id);  
    } else if (JAVADOC_TYPE.equals(type)) {
      artefact = new JavaDoc(id);
    } else if (JUNIT_REPORT_TYPE.equals(type)) {
      artefact = new JunitReport(id);
    } else {
      artefact =  new Doc(id);
    }

    artefact.setName(name);

    DescriptionXmlConverter descrXml = new DescriptionXmlConverter();
    Element descrElement = ele.getChild(descrXml.getElementName());
    if (descrXml.canLoad(descrElement)) {
      artefact.setDescription((Description)descrXml.load(descrElement));    
    }

    Element locationElement = ele.getChild(ELEMENT_LOCATION);
    if (locationElement != null) {
      artefact.setLocation(new Location(locationElement.getTextTrim()));  
    }

    return artefact;    
  }


  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  protected String getContentElementName() {
    return ELEMENT_CONTENT;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  protected boolean canLoadContent(Element ele) {
    
    if (ele == null) {
      return false;
    }

    return getContentElementName().equals(ele.getName());
  }

}
