package ca.aura.xml;

import java.util.Iterator;

import org.jdom.Attribute;
import org.jdom.Element;

import ca.aabeka.util.Debug;
import ca.aabeka.util.Node;
import ca.aabeka.xml.ObjectXmlConverter;
import ca.aura.artefacts.AbstractArtefact;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.sorting.Data;
import ca.aura.artefacts.sorting.Folder;
import ca.aura.project.Depot;
import ca.aura.project.Structure;

/**
 * 
 * @author aforward
 *
 */
public class StructureXmlConverter implements ObjectXmlConverter {

  private static final String ELEMENT_STRUCTURE = "structure";
  private static final String ELEMENT_FOLDER = "folder";
  private static final String ELEMENT_DATA = "data";  

  private static final String ATTRIBUTE_ID = "id";  
  private static final String ATTRIBUTE_NAME = "name";  

  private Depot depot_;
  
  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */
  
  public StructureXmlConverter() {
    init(new Depot());  
  }

  /**
   * 
   * specify the depot from which this structure is created
   * 
   */

  public StructureXmlConverter(Depot depot) {
    init(depot);  
  }

  private void init(Depot depot) {
    depot_ = depot;  
  }


  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#getElementName()
   */
  public String getElementName() {
    return ELEMENT_STRUCTURE;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canLoad(Element)
   */
  public boolean canLoad(Element ele) {
    
    if (ele == null) {
      return false;
    }

    return getElementName().equals(ele.getName());
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#canBuild(Object)
   */
  public boolean canBuild(Object obj) {
    return (obj instanceof Structure);
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#load(Element)
   */
  public Object load(Element ele) {

    Structure structure = new Structure();
    
    if (!canLoad(ele)) {
      Debug.defaultPrintln(this, "can't load ?!?: " + ele);
      return structure;  
    }

    Debug.defaultPrintln(this, "ele: " + ele);

    Iterator folders = ele.getChildren(ELEMENT_FOLDER).iterator();
    while (folders.hasNext()) {
      Debug.defaultPrintln(this,"added? " + structure.add(loadFolder((Element)folders.next())));   
    }


    return structure;
  }

  /**
   * @see ca.aabeka.xml.ObjectXmlConverter#build(Object)
   */
  public Element build(Object obj) {

      
    Element ele = new Element(getElementName());
    Debug.defaultPrintln(this,"building: " + obj.getClass());  
    if (!canBuild(obj)) {
      return ele;  
    }
    
    Structure structure = (Structure)obj;
    Debug.defaultPrintln(this,"building: " + structure.getAllChildren());
    
    
    Iterator children = structure.getChildrenIterator();
    Debug.defaultPrintln(this,"building: " + children.hasNext());    
    
    while (children.hasNext()) {
      
      Folder folder = (Folder)children.next();
      
      Element folderElement = buildFolder(folder);
      Debug.defaultPrintln(this,"building: " + folder.getName());    
      //Debug.defaultPrintln(this, "Justing created folder: " + folder.getAttribute(ATTRIBUTE_NAME).getValue());
      ele.addContent(folderElement);  
    }

    return ele;
  }


  //`````````````````````
  // protected
  //`````````````````````

  /**
   * 
   * load a folder
   * 
   */
  
  protected Folder loadFolder(Element ele) {

    Folder folder = new Folder();

    if (ele == null || !ele.getName().equals(ELEMENT_FOLDER)) {
      return folder;  
    }

    Attribute attr = ele.getAttribute(ATTRIBUTE_NAME);
    
    if (attr != null) {
      folder.setName(attr.getValue()); 
      Debug.defaultPrintln(this,"setting folder: " + attr.getValue());     
    }
    
    addNodes(folder,ele);

    return folder;
    
  }

  /**
   * 
   * load the data
   * 
   */
  
  public Data loadData(Element ele) {
    
    Data data = new Data();
    
    if (ele == null || !ele.getName().equals(ELEMENT_DATA)) {
      return data;
    }

    Attribute attr = ele.getAttribute(ATTRIBUTE_ID);
    assert(attr != null);    
    
    String id = attr.getValue();
    Artefact artefact = depot_.isArtefact(id) ? depot_.getArtefact(id) : new AbstractArtefact(id);
    data.setData(artefact);
    addNodes(data,ele);
    return data;
  }
  
  /**
   * 
   * add the children
   * 
   */
 
  public void addNodes(Node parent, Element ele) {

    Iterator iter;
    
    iter = ele.getChildren(ELEMENT_FOLDER).iterator();
    while (iter.hasNext()) {
      parent.add(loadFolder((Element)iter.next()));     
    }

    iter = ele.getChildren(ELEMENT_DATA).iterator();
    while (iter.hasNext()) {
      parent.add(loadData((Element)iter.next()));     
    }    
    
  }

  /**
   * 
   * build a folder
   * 
   */
  public Element buildFolder(Folder folder) {
    Element ele = new Element(ELEMENT_FOLDER);
    ele.setAttribute(ATTRIBUTE_NAME,folder.getName());
    
    
    addChildren(ele,folder.getChildrenIterator());
    return ele;
  }

  /**
   * 
   * build data
   * 
   */
  public Element buildData(Data data) {
    Element ele = new Element(ELEMENT_DATA);
    
    Artefact artefact = data.getData();
    ele.setAttribute(ATTRIBUTE_ID,artefact.getId());
    
    addChildren(ele,data.getChildrenIterator());
    return ele;
  }

  /**
   * 
   * build a folder or an Element
   * 
   */

  public void addChildren(Element parent, Iterator children) {
    while (children.hasNext()) {
      Object child = children.next();

      if (child instanceof Folder) {
        parent.addContent(buildFolder((Folder)child));  
        continue;
      }  

      if (child instanceof Data) {
        parent.addContent(buildData((Data)child));  
        continue;
      }
    }    
  }


}
