package ca.aura.algorithms;

import java.util.Date;
import java.util.Iterator;

import ca.aabeka.math.NormalDistribution;
import ca.aabeka.util.DateAndTime;
import ca.aabeka.util.Debug;
import ca.aabeka.util.Graph;
import ca.aabeka.util.Hashtable;
import ca.aabeka.util.Vector;
import ca.aura.artefacts.Artefact;
import ca.aura.artefacts.reports.Report;

/**
 * 
 * @author aforward
 *
 */
public class ReferentialIntegrity implements AuraAlgorithm {

  private static final int DEFAULT_K = 20;
 
  private Hashtable ri_;

  //`````````````````````
  // constructor
  //`````````````````````

  /**
   * 
   * default
   * 
   */
  
  public ReferentialIntegrity(Report report, Graph graph) {
    init(report,graph, DEFAULT_K);  
  }

  /**
   * 
   * also set the convergence number
   * 
   */
  
  public ReferentialIntegrity(Report report, Graph graph, int k) {
    k = Math.abs(k);
    init(report,graph, k);  
  }


  private void init(Report report, Graph graph, int k) {
    Hashtable previous = buildInitialHash(report);
    Hashtable  current = (Hashtable)previous.clone();
    
    for (int i=0; i<k; i++) {
      Iterator points;
      points = graph.points();
      while (points.hasNext()) {
        Artefact a = (Artefact)points.next();
        double refIntrity = updateReferenceIntegrity(a,report, graph,previous);
        current.put(a, new Double(refIntrity));
      } 
      
      normalize(current);
      Debug.defaultPrintln(this,"the current: " + current);
      previous = (Hashtable)current.clone();
    }

    ri_ = current;
  }

  //`````````````````````
  // inteface
  //`````````````````````

  /**
   * 
   * @see ca.aura.algorithms.RateAlgorithm#getResults()
   * 
   */
  
  public Hashtable getResults() {
    return ri_;  
  }

  //`````````````````````
  // protected
  //`````````````````````


  /**
   * 
   * update the referential integrity for a particular point
   * 
   * get all q that points to p
   * 
   * q1 -> p
   * q2 -> p
   * q3 -> p
   * 
   * and all r that p points to
   * 
   * r1 -> p
   * r2 -> p
   * 
   * 
   * then apply the following:
   * 
   * RI(q1)*Age(p,q1) + ...
   * 
   * 
   */

  protected double updateReferenceIntegrity(Artefact a, Report report, Graph g, Hashtable ri) {
    double integrity = 0.0;
    integrity += getReferentialContribution(a, g.inReferenceesTo(a),report, g, ri);
    integrity += getReferentialContribution(a,g.outReferencesFrom(a),report, g, ri);
    return integrity;
  }
  

  /**
   * 
   * get the referential integrity of a set of points
   * 
   */
  
  protected double getReferentialContribution(Artefact a, Iterator points, Report report, Graph g, Hashtable ri) {
    double contribution = 0.0;
    while (points.hasNext()) {
      Object p = points.next();
      double integrity = ((Double)ri.get(p)).doubleValue();
      double referenceAge = getReferentialAge(a,(Artefact)p,report);
      contribution += integrity * referenceAge;
    }
    return contribution;
  }

  /**
   * 
   * get the reference age between two poitns
   * 
   */
  
  protected double getReferentialAge(Artefact a1, Artefact a2, Report report) {
    Date d1 = report.getLastUsage(a1).getDate();
    Date d2 = report.getLastUsage(a2).getDate();
    
    Date start = (d1.before(d2)) ? d1 : d2;
    Date end = (d1.after(d2)) ? d1 : d2;
    double referenceAge = DateAndTime.getAgeInDays(start,end);

    return Math.abs(referenceAge);
  }


  /**
   * 
   * build the initial hashtables for the hub and authority values
   * 
   */
  protected Hashtable buildInitialHash(Report r) {
    
    Hashtable hash = new Hashtable();
    
    Iterator points = r.getArtefacts();
    while (points.hasNext()) {
      hash.put(points.next(), new Double(1.0));    
    }  
    return hash;
  }

 /**
   * 
   * normalize by z-score... then shift to avoid negative numbers
   * 
   */
  
  protected void normalize(Hashtable hash) {
    
    Vector values = new Vector(hash.getValues());
    NormalDistribution ndist = new NormalDistribution(values);

    double zScoreShiftValue=0;
   
    Iterator iter;

    iter = hash.getKeys();
    while (iter.hasNext()) {
      Object key = iter.next();
      double x = ((Double)hash.get(key)).doubleValue();
      double newX = ndist.normalize(x);
      zScoreShiftValue = newX < zScoreShiftValue ? newX : zScoreShiftValue;
      hash.put(key, new Double(newX));  
    }

    iter = hash.getKeys();
    while (iter.hasNext()) {
      Object key = iter.next();
      double x = ((Double)hash.get(key)).doubleValue();
      double shiftedX = x + Math.abs(zScoreShiftValue);
      hash.put(key, new Double(shiftedX));  
    }    
  }


}
